@@ -2,7 +2,7 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 
-from sql import Union, Join, Select, Table, Null
+from sql import Union, Join, Select, Table, Null, As
 from sql.conditionals import Greatest
 
 from trytond.pool import PoolMeta, Pool
@@ -24,8 +24,7 @@ shelf_life_delay = fields.TimeDelta(
     help="The delay before removal from the forecast.")
 
 
-class Configuration:
-    __metaclass__ = PoolMeta
+class Configuration(metaclass=PoolMeta):
     __name__ = 'stock.configuration'
 
     shelf_life_delay = fields.MultiValue(shelf_life_delay)
@@ -44,8 +43,7 @@ class ConfigurationLotShelfLife(ModelSQL, ValueMixin):
     shelf_life_delay = shelf_life_delay
 
 
-class Lot:
-    __metaclass__ = PoolMeta
+class Lot(metaclass=PoolMeta):
     __name__ = 'stock.lot'
 
     shelf_life_expiration_date = fields.Date('Shelf Life Expiration Date',
@@ -146,8 +144,7 @@ class Lot:
                         })
 
 
-class Move:
-    __metaclass__ = PoolMeta
+class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
 
     @classmethod
@@ -243,8 +240,9 @@ class Move:
         today = Date.today()
 
         stock_date_end = context.get('stock_date_end') or datetime.date.max
-        if query and ((stock_date_end == today and context.get('forecast'))
-                or stock_date_end > today):
+        if (query and not context.get('skip_lot_sled')
+                and ((stock_date_end == today and context.get('forecast'))
+                    or stock_date_end > today)):
             lot = Lot.__table__()
 
             config = Config(1)
@@ -277,10 +275,9 @@ class Move:
                 elif isinstance(query, Select):
                     yield query
 
-            union, = query.from_
-            for sub_query in find_queries(union):
+            def add_join(from_):
                 # Find move table
-                for i, table in enumerate(sub_query.from_):
+                for i, table in enumerate(from_):
                     if isinstance(table, Table) and table._name == cls._table:
                         sub_query.from_[i] = join(table)
                         break
@@ -291,14 +288,43 @@ class Move:
                         break
                 else:
                     # Not query on move table
-                    continue
-                sub_query.where &= ((lot.shelf_life_expiration_date == Null)
-                    | (lot.shelf_life_expiration_date >= expiration_date))
+                    return False
+                return True
+
+            union, = query.from_
+            for sub_query in find_queries(union):
+                if add_join(sub_query.from_):
+                    sub_query.where &= (
+                        (lot.shelf_life_expiration_date == Null)
+                        | (lot.shelf_life_expiration_date >= expiration_date))
+
+            stock_date_start = context.get('stock_date_start')
+            if stock_date_start:
+                with Transaction().set_context(
+                        stock_date_start=None,
+                        skip_lot_sled=True):
+                    query_expired = cls.compute_quantities_query(
+                        location_ids, with_childs=with_childs,
+                        grouping=grouping, grouping_filter=grouping_filter)
+                    union_expired, = query_expired.from_
+                    for sub_query in find_queries(union_expired):
+                        if add_join(sub_query.from_):
+                            # only lot expiring during the period
+                            sub_query.where &= (
+                                (lot.shelf_life_expiration_date >=
+                                    stock_date_start)
+                                & (lot.shelf_life_expiration_date <
+                                    stock_date_end))
+                        # Inverse quantity
+                        for column in sub_query._columns:
+                            if (isinstance(column, As)
+                                    and column.output_name == 'quantity'):
+                                column.expression = -column.expression
+                    union.queries += union_expired.queries
         return query
 
 
-class Period:
-    __metaclass__ = PoolMeta
+class Period(metaclass=PoolMeta):
     __name__ = 'stock.period'
 
     @classmethod
@@ -340,12 +366,8 @@ class Period:
         if lot_id:
             lot_id, = lot_id
             lot = Lot(lot_id)
-            lang, = Lang.search([
-                    ('code', '=', Transaction().language),
-                    ])
-            date = Lang.strftime(
-                lot.shelf_life_expiration_date, lang.code, lang.date)
+            lang = Lang.get()
             cls.raise_user_error('close_period_sled', {
-                    'date': date,
+                    'date': lang.strftime(lot.shelf_life_expiration_date),
                     'lot': lot.rec_name,
                     })
